import { getNameOfServiceType } from "../helpers/getNameOfServiceType.js";
import { IServiceProvider } from "./IServiceProvider.js";
import { ServiceLifetime } from "./ServiceLifetime.js";
import { ServiceRegistry } from "./ServiceRegistry.js";
import { isServiceImplementationResolver } from "../implementation-resolvers/BaseServiceImplementationResolver.js";
import { RegistryServiceImplementationResolver } from "../implementation-resolvers/RegistryServiceImplementationResolver.js";
import { ValueServiceImplementationResolver } from "../implementation-resolvers/ValueServiceImplementationResolver.js";
export class ServiceCollection {
    collection;
    constructor(collection) {
        this.collection = new Map(collection?.copy() || []);
        if (!this.collection.has(IServiceProvider)) {
            this.addTransient(IServiceProvider);
        }
    }
    *[Symbol.iterator]() {
        for (const descriptors of this.collection.values()) {
            for (const descriptor of descriptors) {
                yield descriptor;
            }
        }
    }
    getDescriptors(service) {
        return (this.collection.get(service) ?? []);
    }
    getDescriptor(service) {
        const descriptors = this.getDescriptors(service);
        if (descriptors.length === 0) {
            let name = getNameOfServiceType(service);
            const metadata = ServiceRegistry.get(service);
            if (metadata?.name) {
                name = metadata.name;
            }
            throw new Error(`No service of type "${name}" is registered`);
        }
        return descriptors[descriptors.length - 1];
    }
    addScoped(service, implementation = service) {
        this.addService(ServiceLifetime.Scoped, service, implementation);
        return this;
    }
    addTransient(service, implementation = service) {
        this.addService(ServiceLifetime.Transient, service, implementation);
        return this;
    }
    addSingleton(service, implementation = service) {
        this.addService(ServiceLifetime.Singleton, service, implementation);
        return this;
    }
    *copy() {
        for (const [key, descriptors] of this.collection) {
            yield [key, [...descriptors.map((d) => ({ ...d }))]];
        }
    }
    addService(lifetime, service, implementation) {
        let resolver;
        if (isServiceImplementationResolver(implementation)) {
            resolver = implementation;
        }
        else if (typeof implementation === "function") {
            resolver = new RegistryServiceImplementationResolver(implementation);
        }
        else {
            resolver = new ValueServiceImplementationResolver(implementation);
        }
        const descriptors = this.collection.get(service) ?? [];
        this.collection.set(service, [
            ...descriptors,
            {
                service,
                lifetime,
                resolver,
            },
        ]);
        return this;
    }
}
//# sourceMappingURL=ServiceCollection.js.map