import { IServiceProvider } from "./IServiceProvider.js";
import { ServiceCollection } from "./ServiceCollection.js";
import { ServiceInstancesStore } from "./ServiceInstancesStore.js";
import { ServiceLifetime } from "./ServiceLifetime.js";
import { validateRequestPath } from "./validation/validateRequestPath.js";
import { getNameOfDescriptor } from "../helpers/getNameOfDescriptor.js";
import { resolveGeneratorAsync } from "../helpers/resolveGeneratorAsync.js";
import { resolveGeneratorSync } from "../helpers/resolveGeneratorSync.js";
import { isServiceProvider } from "../helpers/isServiceProvider.js";
import { all } from "../service-type-resolvers/all.js";
import { single } from "../service-type-resolvers/single.js";
import { isServiceTypeResolver } from "../service-type-resolvers/BaseServiceTypeResolver.js";
export class ServiceProvider {
    collection;
    parent;
    static internalGetService(provider, service, requestedBy, mode) {
        if (!(provider instanceof ServiceProvider)) {
            throw new Error("provider must be an instance of ServiceProvider");
        }
        return provider.internalGetService(service, requestedBy, mode);
    }
    static getDescriptor(provider, service) {
        if (!(provider instanceof ServiceProvider)) {
            throw new Error("provider must be an instance of ServiceProvider");
        }
        return provider.collection.getDescriptor(service);
    }
    static getDescriptors(provider, service) {
        if (!(provider instanceof ServiceProvider)) {
            throw new Error("provider must be an instance of ServiceProvider");
        }
        return provider.collection.getDescriptors(service);
    }
    instancesStore;
    constructor(collection, parent) {
        this.collection = collection;
        this.parent = parent;
        this.instancesStore = new ServiceInstancesStore();
        this.internalGetService = this.internalGetService.bind(this);
        this.resolveServiceImplementation =
            this.resolveServiceImplementation.bind(this);
    }
    getServices(service) {
        return resolveGeneratorSync(this.internalGetService(all(service), new Set(), "sync"));
    }
    getServiceAsync(service) {
        if (!isServiceTypeResolver(service)) {
            service = single(service);
        }
        return resolveGeneratorAsync(this.internalGetService(service, new Set(), "async"));
    }
    getService(service) {
        if (!isServiceTypeResolver(service)) {
            service = single(service);
        }
        return resolveGeneratorSync(this.internalGetService(service, new Set(), "sync"));
    }
    getServicesAsync(service) {
        return resolveGeneratorAsync(this.internalGetService(all(service), new Set(), "async"));
    }
    createScope() {
        const serviceProvider = new ServiceProvider(this.collection, this);
        return {
            serviceProvider,
            [Symbol.dispose]: () => {
                serviceProvider[Symbol.dispose]();
            },
        };
    }
    createAsyncScope() {
        const serviceProvider = new ServiceProvider(this.collection, this);
        return {
            serviceProvider,
            [Symbol.asyncDispose]: async () => {
                await serviceProvider[Symbol.asyncDispose]();
            },
        };
    }
    *internalGetService(service, requestedBy, mode) {
        return yield* service.resolve(this.collection, this.instancesStore, this.resolveServiceImplementation, requestedBy, mode);
    }
    *resolveServiceImplementation(descriptor, requestedBy, mode) {
        if (!this.instancesStore.getInstanceInfo(descriptor)?.initialized) {
            validateRequestPath(requestedBy, descriptor);
        }
        if (descriptor.lifetime === ServiceLifetime.Singleton && this.parent) {
            return yield* this.parent.resolveServiceImplementation(descriptor, requestedBy, mode);
        }
        return yield* this.createInstanceFromDescriptor(descriptor, requestedBy, mode);
    }
    *createInstanceFromDescriptor(descriptor, requestedBy, mode) {
        try {
            const lastRequestedBy = [...requestedBy].pop();
            if (descriptor.lifetime === ServiceLifetime.Scoped) {
                if (lastRequestedBy?.lifetime === ServiceLifetime.Singleton) {
                    throw new Error(`Scoped service cannot be resolved from singleton service.`);
                }
                if (!this.parent) {
                    throw new Error("Scoped services require a service scope.");
                }
            }
            switch (descriptor.lifetime) {
                case ServiceLifetime.Scoped:
                case ServiceLifetime.Singleton: {
                    const instanceInfo = this.instancesStore.addInstance(descriptor, lastRequestedBy);
                    instanceInfo.initialize(yield* descriptor.resolver.resolve(this.internalGetService, descriptor, requestedBy, mode));
                    return instanceInfo.instance;
                }
                case ServiceLifetime.Transient: {
                    if (isServiceProvider(descriptor.service)) {
                        return this;
                    }
                    const implementationGetter = yield* descriptor.resolver.resolve(this.internalGetService, descriptor, requestedBy, mode);
                    return descriptor.dry ? null : implementationGetter();
                }
            }
        }
        catch (exception) {
            throw new Error(`Failed to initiate service ${getNameOfDescriptor(descriptor)}:\n\r${exception.message}`, { cause: exception });
        }
    }
    [Symbol.dispose]() {
        this.instancesStore[Symbol.dispose]();
    }
    async [Symbol.asyncDispose]() {
        await this.instancesStore[Symbol.asyncDispose]();
    }
}
//# sourceMappingURL=ServiceProvider.js.map